function  result  = madDownloadFile(cgiurl,  fullFilename, outputFile, user_fullname, user_email, user_affiliation, format )
%madDownloadFile downloads a Madrigal file to local computer in various
%formats
%
%    The calling syntax is:
%       result  = madDownloadFile(cgiurl,  fullFilename, outputFile, [ format ] )
%   
% Inputs: 
%
%      1. cgiurl (string) to Madrigal site cgi directory 
%        (Example: 'http://madrigal.haystack.mit.edu/cgi-bin/madrigal/') 
%         Note that method getMadrigalCgiUrl converts homepage url into
%         cgiurl. 
%
%      2. fullFilename - file to download as returned by getExperimentFilesWeb.m
%
%      3. outputFile - name to save new file to
%
%      4. user_fullname - is user name (string)
%
%      5. user_email - is user email address (string)
%
%      6. user_affiliation - is user affiliation (string) 
%
%      7. format - one of the following strings:
%          'simple', 'hdf5', 'cedar_ascii'
%              'simple' is the default if not specified.  It
%              is recommended that users choose either 'simple' or 'hdf5'
%              'simple' is an ascii space delimited column 
%              format.  See Cedar
%              standard for description of complex cedar_ascii format
%              (not recommended)
%
%         'hdf5 option will only work with Madrigal 2.6 or greater'
%
%    $Id: madDownloadFile.m 4429 2014-07-30 18:53:09Z brideout $
%
result = -1;

if (nargin < 6)
    error('Usage: result  = madDownloadFile(cgiurl,  fullFilename, outputFile, user_fullname, user_email, user_affiliation, [ format ] )');
end
if (nargin < 7)
    format = 'simple';
end

if (strcmp(format, 'simple'))
    fileType = -1;
elseif (strcmp(format, 'hdf5'))
    % fileType -3 is base64 encoded hdf5 - binary downloads just don't seem
    % to work
    fileType = -3;
    % verify Madrigal can handle this
    thisVer = getVersion(cgiurl);
    if (compareVersions(thisVer, '2.6') < 0)
        err.message = 'This madrigal site does not support hdf5 format until it is upgraded to at least 2.6';
        err.identifier = 'madmatlab:badArguments';
        rethrow(err);
    end
elseif (strcmp(format, 'cedar_ascii'))
    fileType = 4;
else
    err.message = 'illegal format argument in madDownloadFile';
    err.identifier = 'madmatlab:badArguments';
    rethrow(err);
end

% build the complete cgi url
if (cgiurl(end) == '/')
    cgiurl = strcat(cgiurl, 'getMadfile.cgi?');
else
    cgiurl = strcat(cgiurl, '/getMadfile.cgi?');
end


% append filename and filetype
[temp  errmsp] = sprintf('fileName=%s&fileType=%i&user_fullname=%s&user_email=%s&user_affiliation=%s', ...
    fullFilename, fileType, user_fullname, user_email, user_affiliation);
cgiurl = strcat(cgiurl, temp);

% make sure any + replaced by %2B
cgiurl = strrep(cgiurl,'+','%2B');

% make sure any space replaced by +
cgiurl = strrep(cgiurl,' ','+');

wget_command = horzcat('wget -q -O', ' ', outputFile, ' "', cgiurl, '"');
% for wget do not use -3 for Hdf5, because it can handle binary
wget_command = strrep(wget_command, '-3', '-2');

% first try to use wget
[result, cmdout] = system(wget_command);
if (result == 0)
    return
else
    warning('wget may not be installed, less robust Matlab urlread will be used instead');
end

% now get that url
data = urlread(cgiurl);

if (fileType == -3)
    data = base64decode(data);
end

% look for errors - if html returned, error occurred
htmlList = strfind(data, '</html>');
if (~isempty(htmlList))
    err.message = strcat('Unable to run cgi script madDownloadFile using cgiurl: - ', cgiurl);
    err.identifier = 'madmatlab:scriptError';
    rethrow(err);
end

% surpress matlab warning about multibyte Characters
warning off REGEXP:multibyteCharacters

f = fopen(outputFile, 'w');
fwrite(f, data);
fclose(f);

result = 0;
end

